﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;

namespace Inedo.Consolation
{
    internal abstract class ConsoleOption
    {
        protected ConsoleOption(ICustomAttributeProvider source)
        {
            this.Name = ((string)((dynamic)source).Name).Replace('_', '-');

            this.Aliases = new HashSet<string>(
                from a in source.GetCustomAttributes(typeof(ConsoleAliasAttribute), true).Cast<ConsoleAliasAttribute>()
                select a.Alias,
                StringComparer.OrdinalIgnoreCase
            );

            var descriptionAttribute = (DescriptionAttribute)Attribute.GetCustomAttribute((dynamic)source, typeof(DescriptionAttribute));
            if (descriptionAttribute != null)
                this.Description = descriptionAttribute.Description;
        }

        public string Name { get; }
        public HashSet<string> Aliases { get; }
        public string Description { get;  }
        public bool IsFlag { get; protected set; }
        public object DefaultValue { get; protected set; }
        public abstract object CurrentValue { get; }

        public abstract void SetValue(string valueText);
        public abstract void SetFlag(bool enableDisable);

        protected object ParseValue(string valueText, Type targetType)
        {
            if (targetType == typeof(string))
                return valueText;

            var type = Nullable.GetUnderlyingType(targetType) ?? targetType;
            try
            {
                if (type.IsEnum)
                    return Enum.Parse(type, valueText, true);
                else
                    return Convert.ChangeType(valueText, type);
            }
            catch
            {
                throw new ConsoleArgumentException(string.Format("Invalid value for {0}: Could not convert \"{1}\" to a valid {2}", this.Name, valueText, type.Name));
            }
        }
    }
}
