﻿using System;
using System.Diagnostics;
using System.IO;
using System.IO.Compression;
using Inedo.Agents.Service;

namespace Inedo.Agents.Hosting
{
    internal sealed class HostedProcessManager : SlimHostedProcessManager
    {
        private readonly VirtualAgentHost[] hosts = new VirtualAgentHost[2];

        public HostedProcessManager()
        {
            this.hosts[(int)AgentType.Otter - 1] = new VirtualAgentHost(AgentType.Otter, AgentConfig.Otter);
            this.hosts[(int)AgentType.BuildMaster - 1] = new VirtualAgentHost(AgentType.BuildMaster, AgentConfig.BuildMaster);
        }

        public event EventHandler<CommandResponseEventArgs> ResponseReceived;

        public new static HostedProcessManager Instance
        {
            get { return (HostedProcessManager)SlimHostedProcessManager.Instance; }
            set { SlimHostedProcessManager.Instance = value; }
        }

        public override void InstallAgentUpdate(Commands.UpdateAgentCommand update)
        {
            if (update.AgentType == AgentType.Meta)
            {
                var servicePath = Path.GetDirectoryName(typeof(Program).Assembly.Location);
                var tempPath = Path.Combine(servicePath, "_tmp");
                Directory.CreateDirectory(tempPath);

                foreach (var ass in update.Assemblies)
                {
                    var targetFileName = Path.Combine(tempPath, ass.Name);

                    using (var sourceStream = new GZipStream(ass.Data, CompressionMode.Decompress))
                    using (var targetFileStream = File.Create(targetFileName))
                    {
                        sourceStream.CopyTo(targetFileStream);
                    }

                    File.SetLastWriteTimeUtc(targetFileName, ass.LastWriteTime);
                }

                int pid;
                using (var currentProcess = Process.GetCurrentProcess())
                {
                    pid = currentProcess.Id;
                }

                var updatePath = Path.Combine(tempPath, "InedoAgentService.exe");
                using (SlimProcess.Start(updatePath, $"update \"--targetPath={servicePath}\" \"--serviceName={AgentService.Instance.Name}\" --pid={pid}"))
                {
                }
            }
            else
            {
                this.GetVirtualHost(update.AgentType).InstallAgentUpdate(update);
            }
        }
        public VirtualAgentHost GetVirtualHost(AgentType agentType) => this.hosts[(int)agentType - 1];
        public void SessionDisconnected(int sessionId)
        {
            foreach (var host in this.hosts)
                host.DisconnectSession(sessionId);
        }
        public void OnResponseReceived(CommandResponseEventArgs e) => this.ResponseReceived?.Invoke(this, e);
    }
}
